@@ -23,20 +23,21 @@ loaded, and receives each message for the specified topic through the
 :meth:`Consumer.consume` method.
 
 .. moduleauthor:: Luke Macken <lmacken@redhat.com>
+.. moduleauthor:: Ralph Bean <rbean@redhat.com>
 """
 
 import json
 import threading
+import time
 import logging
 log = logging.getLogger('moksha.hub')
 
-try:
-    import queue  # py3
-except ImportError:
-    import Queue as queue  # py2
+import six.moves.queue as queue
+from collections import deque
 
 from kitchen.iterutils import iterate
 from moksha.common.lib.helpers import create_app_engine
+from moksha.common.lib.converters import asbool
 import moksha.hub.reactor
 
 
@@ -59,6 +60,8 @@ class Consumer(object):
         # receiving raw messages, and a worker thread that pulls items off
         # the queue to do "consume" work.
         self.incoming = queue.Queue()
+        self.headcount_in = self.headcount_out = 0
+        self._times = deque(maxlen=1024)
 
         callback = self._consume
         if self.jsonify:
@@ -78,20 +81,32 @@ class Consumer(object):
             self.engine = create_app_engine(app, hub.config)
             self.DBSession = sessionmaker(bind=self.engine)()
 
-        self.N = int(self.hub.config.get('moksha.workers_per_consumer', 1))
-        for i in range(self.N):
-            moksha.hub.reactor.reactor.callInThread(self._work)
+        self.blocking_mode = asbool(self.hub.config.get('moksha.blocking_mode', False))
+        if self.blocking_mode:
+            log.info("Blocking mode true for %r.  "
+                     "Messages handled as they arrive." % self)
+        else:
+            self.N = int(self.hub.config.get('moksha.workers_per_consumer', 1))
+            log.info("Blocking mode false for %r.  "
+                     "Messages to be queued and distributed to %r threads." % (
+                         self, self.N))
+            for i in range(self.N):
+                moksha.hub.reactor.reactor.callInThread(self._work_loop)
 
         self._initialized = True
 
     def __json__(self):
-
         if self._initialized:
             backlog = self.incoming.qsize()
+            headcount_out = self.headcount_out
+            headcount_in = self.headcount_in
+            times = list(self._times)
         else:
             backlog = None
+            headcount_out = headcount_in = 0
+            times = []
 
-        return {
+        results = {
             "name": type(self).__name__,
             "module": type(self).__module__,
             "topic": self.topic,
@@ -99,7 +114,15 @@ class Consumer(object):
             "exceptions": self._exception_count,
             "jsonify": self.jsonify,
             "backlog": backlog,
+            "headcount_out": headcount_out,
+            "headcount_in": headcount_in,
+            "times": times,
         }
+        # Reset these counters before returning.
+        self.headcount_out = self.headcount_in = 0
+        self._exception_count = 0
+        self._times.clear()
+        return results
 
     def debug(self, message):
         idx = threading.current_thread().ident
@@ -141,54 +164,64 @@ class Consumer(object):
                 # Weird.  I have no idea...
                 pass
 
-        try:
-            message_as_dict = {'body': body, 'topic': topic}
-            self._consume(message_as_dict)
-            self._exception_count = 0  # Reset if everything went swimmingly
-        except Exception:
-            # Otherwise, keep track of how many exceptions we've hit in a row
-            self._exception_count = self._exception_count + 1
-            # And then re-raise the exception to be logged
-            raise
+        message_as_dict = {'body': body, 'topic': topic}
+        return self._consume(message_as_dict)
 
     def _consume(self, message):
-        self.incoming.put(message)
+        self.headcount_in += 1
+        if self.blocking_mode:
+            # Do the work right now
+            return self._do_work(message)
+        else:
+            # Otherwise, put the message in a queue for other threads to handle
+            self.incoming.put(message)
 
-    def _work(self):
+    def _work_loop(self):
         while True:
             # This is a blocking call.  It waits until a message is available.
             message = self.incoming.get()
-
             # Then we are being asked to quit
             if message is StopIteration:
                 break
-
-            self.debug("Worker thread picking a message.")
-            try:
-                self.validate(message)
-            except Exception as e:
-                log.warn("Received invalid message %r" % e)
-                continue
-
-            try:
-                self.pre_consume(message)
-            except Exception as e:
-                self.log.exception(message)
-
-            try:
-                self.consume(message)
-            except Exception as e:
-                self.log.exception(message)
-
-            try:
-                self.post_consume(message)
-            except Exception as e:
-                self.log.exception(message)
-
-            self.debug("Going back to waiting on the incoming queue.")
-
+            self._do_work(message)
         self.debug("Worker thread exiting.")
 
+    def _do_work(self, message):
+        self.headcount_out += 1
+        start = time.time()
+        handled = True
+
+        self.debug("Worker thread picking a message.")
+        try:
+            self.validate(message)
+        except Exception as e:
+            log.warning("Received invalid message %r" % e)
+            return False  # Not handled
+
+        try:
+            self.pre_consume(message)
+        except Exception as e:
+            self.log.exception(message)
+
+        try:
+            self.consume(message)
+        except Exception as e:
+            handled = False  # Not handled.  Return this later.
+            self.log.exception(message)
+            # Keep track of how many exceptions we've hit in a row
+            self._exception_count += 1
+
+        try:
+            self.post_consume(message)
+        except Exception as e:
+            self.log.exception(message)
+
+        # Record how long it took to process this message (for stats)
+        self._times.append(time.time() - start)
+
+        self.debug("Going back to waiting on the incoming queue.  Message handled: %r" % handled)
+        return handled
+
     def validate(self, message):
         """ Override to implement your own validation scheme. """
         pass
